#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#
# This script merges the Delaware and CRU historical climate
# datasets, adding crop area and production, and assigning each
# observation to a 'region.' The region grouping used is by yield
# quartile, with the lowest yielding nations as group 'Q1' and
# the highest as group 'Q4'.
#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#~#


setwd('Z:\\Lobell_Lab')
crops = c('maize','maize2','rice','rice2','wheat','wheat2','soy','barley','barley2','sorghum','sorghum2')
ncrops = length(crops)
df.CRU = read.csv('world_panels\\quantile-grouping\\panels_half_deg_area\\cru.6crop.panel.1952.2002.csv', header=T)
df.UDel = read.csv('world_panels\\quantile-grouping\\panels_half_deg_area\\udel.6crop.panel.1959.2008.csv', header=T)
ccodes = read.csv('world_panels\\cntry.codes.all.csv', header=T)
countries = union(unique(df.CRU$FID), unique(df.UDel$FID))
cntryIDs = countries[order(countries)]
cntryNames = ccodes[match(cntryIDs, ccodes[,1]),'gmi_cntry_name']
faoCodes = ccodes[match(cntryIDs, ccodes[,1]),'fao_code']
ncntry = length(countries)
years = 1961:2008
nyr = length(years)
startYr = years[1]
endYr = years[nyr]
df.merged = data.frame(array(dim=c(ncntry*nyr*ncrops,5)))
colnames(df.merged) = c('Country', 'FID', 'FAO_code', 'Year', 'Crop')
df.merged$Country = rep(cntryNames, rep(nyr*ncrops,ncntry))
df.merged$FID = rep(cntryIDs, rep(nyr*ncrops,ncntry))
df.merged$FAO_code = rep(faoCodes,rep(nyr*ncrops,ncntry))
df.merged$Year = rep(rep(years,rep(ncrops,nyr)),ncntry)
df.merged$Crop = rep(crops, length(cntryNames)*nyr)
df.merged = merge(df.merged, df.CRU, all.x=T, by.x=c('FID', 'Year', 'Crop'), by.y=c('FID', 'Year', 'Crop'))
df.merged = df.merged[,-c(6:7)]
df.merged$tAvgCRU = (df.merged[,7] + df.merged[,8])/2
df.merged = merge(df.merged, df.UDel, all.x=T, by.x=c('FID', 'Year', 'Crop'), by.y=c('FID', 'Year', 'Crop'))
df.merged = df.merged[,-c(10:11)]
colnames(df.merged) = c('FID', 'Year', 'Crop', 'Country', 'FAO_code', 'precCRU',
                        'tMinCRU', 'tMaxCRU', 'tAvgCRU', 'tAvgUDel', 'precUDel')

### Multiply everything from CRU set by 10
df.merged[,c('precCRU', 'tMinCRU', 'tMaxCRU', 'tAvgCRU')] = 10*df.merged[,c('precCRU', 'tMinCRU', 'tMaxCRU', 'tAvgCRU')]
df.merged = df.merged[-which(is.na(df.merged$FAO_code)),]

### Add FAO yields
rice.yield = read.csv('FAO_stats\\rice-yields-61-08.csv', header=T)
maize.yield = read.csv('FAO_stats\\maize-yields-61-08.csv', header=T)
wheat.yield = read.csv('FAO_stats\\wheat-yields-61-08.csv', header=T)
soybean.yield = read.csv('FAO_stats\\soybean-yields-61-08.csv', header=T)
barley.yield = read.csv('FAO_stats\\barley-yields-61-08.csv', header=T)
sorghum.yield = read.csv('FAO_stats\\sorghum-yields-61-08.csv', header=T)
yieldList = list(maize.yield, maize.yield, rice.yield, rice.yield, wheat.yield,
                 wheat.yield, soybean.yield, barley.yield, barley.yield,
                 sorghum.yield, sorghum.yield)
names(yieldList) = c('maize','maize2','rice','rice2','wheat','wheat2',
                     'soy','barley','barley2','sorghum','sorghum2')
df.merged$Yield = sapply(1:nrow(df.merged), function(i) {
  if(any(yieldList[[df.merged$Crop[i]]]$country.codes == df.merged$FAO_code[i]))
    return(yieldList[[df.merged$Crop[i]]][which(yieldList[[df.merged$Crop[i]]]$country.codes
                      == df.merged$FAO_code[i]), paste('X',df.merged$Year[i],sep='')])
  else
    return(NA)})

### Add FAO production
rice.prod = read.csv('FAO_stats\\rice-production-61-08.csv', header=T)
maize.prod = read.csv('FAO_stats\\maize-production-61-08.csv', header=T)
wheat.prod = read.csv('FAO_stats\\wheat-production-61-08.csv', header=T)
soybean.prod = read.csv('FAO_stats\\soybean-production-61-08.csv', header=T)
barley.prod = read.csv('FAO_stats\\barley-production-61-08.csv', header=T)
sorghum.prod = read.csv('FAO_stats\\sorghum-production-61-08.csv', header=T)
prodList = list(maize.prod, maize.prod, rice.prod, rice.prod, wheat.prod,
                wheat.prod, soybean.prod, barley.prod, barley.prod,
                sorghum.prod, sorghum.prod)
names(prodList) = c('maize','maize2','rice','rice2','wheat','wheat2',
                    'soy','barley','barley2','sorghum','sorghum2')
df.merged$Production = sapply(1:nrow(df.merged), function(i) {
  if(any(prodList[[df.merged$Crop[i]]]$country.codes == df.merged$FAO_code[i]))
    return(prodList[[df.merged$Crop[i]]][which(prodList[[df.merged$Crop[i]]]$country.codes
                      == df.merged$FAO_code[i]), paste('X',df.merged$Year[i],sep='')])
  else
    return(NA)})


### Add FAO area
rice.area = read.csv('FAO_stats\\rice-area-61-08.csv', header=T)
maize.area = read.csv('FAO_stats\\maize-area-61-08.csv', header=T)
wheat.area = read.csv('FAO_stats\\wheat-area-61-08.csv', header=T)
soybean.area = read.csv('FAO_stats\\soybean-area-61-08.csv', header=T)
barley.area = read.csv('FAO_stats\\barley-area-61-08.csv', header=T)
sorghum.area = read.csv('FAO_stats\\sorghum-area-61-08.csv', header=T)
areaList = list(maize.area, maize.area, rice.area, rice.area, wheat.area,
                wheat.area, soybean.area, barley.area, barley.area,
                sorghum.area, sorghum.area)
names(areaList) = c('maize','maize2','rice','rice2','wheat','wheat2',
                    'soy','barley','barley2','sorghum','sorghum2')
df.merged$Area = sapply(1:nrow(df.merged), function(i) {
  if(any(areaList[[df.merged$Crop[i]]]$country.codes == df.merged$FAO_code[i]))
    return(areaList[[df.merged$Crop[i]]][which(areaList[[df.merged$Crop[i]]]$country.codes
                      == df.merged$FAO_code[i]), paste('X',df.merged$Year[i],sep='')])
  else
    return(NA)})

### Function takes a crop as input, returns (and saves) a list of
### countries that produce that crop, ranked by yield, with the
### appropriate region number
RankYields <- function(crop) {
  # we want to use only the general crop name, i.e. not have '2' appended
  cropName <- ifelse(substring(crop, nchar(crop))=='2',
                     substr(crop, 1, nchar(crop)-1),
                     crop)
  cropName2 <- paste(cropName, 2, sep='')
  # get data for specified crop, between 1998 and 2002
  tmpData <- df.merged[which((df.merged$Crop == cropName |
                              df.merged$Crop == cropName2) &
                              df.merged$Year >= 1998 &
                              df.merged$Year <= 2002),]
  # get rid of maize2 data for countries that don't have it
  tmpData <- tmpData[which(!is.na(tmpData$tAvgCRU)),]
  if (length(which(duplicated(tmpData[,c('Country', 'Year')]))) > 0)
    tmpData <- tmpData[-which(duplicated(tmpData[,c('Country', 'Year')])),]
  countries <- unique(tmpData$Country)
  region <- rep(NA, length(countries))
  yield <- sapply(countries, function(x) {
    mean(as.numeric(tmpData[which(tmpData$Country==x),'Yield']),na.rm=T)
  })
  faoCode <- sapply(countries, function(x) {
    tmpData[which(tmpData$Country==x),'FAO_code'][1]
  })
  yield <- data.frame(countries, faoCode, region, yield)
  yield <- yield[order(yield[,'yield'], decreasing=T),]
  colnames(yield) <- c('country', 'fao_code', 'region', 'yield')
  q1.indices <- which(yield$yield <= quantile(yield$yield, 0.25, na.rm=T))
  q2.indices <- which(yield$yield <= quantile(yield$yield, 0.5, na.rm=T) &
                      yield$yield > quantile(yield$yield, 0.25, na.rm=T))
  q3.indices <- which(yield$yield <= quantile(yield$yield, 0.75, na.rm=T) &
                      yield$yield > quantile(yield$yield, 0.5, na.rm=T))
  q4.indices <- which(yield$yield > quantile(yield$yield, 0.75, na.rm=T))
  for (i in 1:4)
    yield[eval(parse(text=paste('q',i,'.indices',sep=''))),'region'] <- i
  write.csv(yield, paste('world_panels\\quantile-grouping\\region-files\\',
                         cropName,'-regions.csv', sep=''), row.names=F)
  yield
}

df.merged <- cbind(df.merged, rep(NA, nrow(df.merged)))
names(df.merged)[ncol(df.merged)] = 'Region'
for (cr in crops) {
  if (!substring(cr, nchar(cr))=='2')
    yields <- RankYields(cr)
  for (country in unique(yields$country)) {
    cntryReg <- yields[which(yields$country==country),'region']
    cntryIndex <- which(df.merged$Crop==cr & df.merged$Country==country)
    df.merged[cntryIndex,'Region'] <- cntryReg
  }
}

### Order the observations in the large panel set, for no particular reason
df.merged = df.merged[order(df.merged$Crop),]
df.merged = df.merged[order(df.merged$Year),]
df.merged = df.merged[order(df.merged$Country),]

df.merged$logYield = log(df.merged$Yield)
write.csv(df.merged, paste('world_panels\\quantile-grouping\\panels_half_deg_area\\udel.cru.panel.6crop.nayield',
          years[1], years[nyr], 'csv',sep='.'), row.names=F)
